package main

import (
	"fmt"
	"launchpad.net/goetveld/rietveld"
	"launchpad.net/lpad"
)

func codeReviewURL(issue *rietveld.Issue) string {
	return rietveld.CodeReview.IssueURL(issue)
}

func sendCodeReview(sourceInfo, targetInfo *BranchInfo, log *Log, mp, mpreq *lpad.MergeProposal) (*rietveld.Issue, error) {
	logf("Diffing branches for codereview: %s => %s", sourceInfo.Path, targetInfo.Path)
	delta, err := rietveld.BazaarDiffBranches(targetInfo.Path, sourceInfo.Path)
	if err != nil {
		return nil, fmt.Errorf("Can't diff branches for codereview: %v", err)
	}
	reqpage := ""
	if mpreq != nil {
		reqpage = "Requires: " + mpreq.WebPage() + "\n\n"
	}
	description := fmt.Sprintf("%s\n\n%s\n\n%s\n\n%s(do not edit description out of merge proposal)\n",
		log.Summary, log.Description, mp.WebPage(), reqpage)
	issue := &rietveld.Issue{Id: log.CodeReviewId}
	creating := issue.Id == 0
	if creating {
		issue.Subject = log.Summary
		issue.Description = description
		issue.ReviewerMails = []string{mp.Email()}
	} else {
		id := issue.Id
		issue, err = rietveld.CodeReview.Issue(id)
		if err != nil {
			return nil, fmt.Errorf("Can't load issue %d from codereview: %v", id, err)
		}
		if issue.Description != description {
			issue.Subject = log.Summary
			issue.Description = description
			err := rietveld.CodeReview.UpdateIssue(issue)
			if err != nil {
				return nil, fmt.Errorf("Can't update issue %d in codereview: %v", issue.Id, err)
			}
		}
		logf("Pushing new patch set to codereview issue %d...", issue.Id)
	}
	if err = rietveld.CodeReview.SendDelta(issue, delta, false); err != nil {
		return nil, fmt.Errorf("Failed to send patch set to codereview: %v", err)
	}
	if creating {
		logf("Created new issue: https://codereview.appspot.com/%d/", issue.Id)
		log.CodeReviewId = issue.Id
	}
	return issue, nil
}

func addCodeReviewComment(issue *rietveld.Issue, message string, unsubscribe bool) error {
	logf("Adding comment to codereview issue...")
	comment := &rietveld.Comment{Message: message, PublishDrafts: true}
	if unsubscribe {
		comment.Reviewers = justMp(issue.ReviewerMails)
		comment.Cc = justMp(issue.CcMails)
	}
	err := rietveld.CodeReview.AddComment(issue, comment)
	if err != nil {
		return fmt.Errorf("Error adding comment to codereview issue: %v", err)
	}
	return nil
}
